% Use of quaternions to numerically solve the problem of the spinning top 
% The top is released with an angular velocity of 1000 revolutions per 
% minute(rpm) around its axis of symmetry (the body z axis), which makes 
% an angle of 60  with the vertical (the inertial Z axis).
% Top properties 
clc;
clear all;
 m  = 0.5;      % [kg]
 A = 12e-4;     % [kg*m^2] A = Ix = Iy
 C = 4.5e-4;    % [kg*m^2] C = Iz
 d = 0.05;      % [m]  distance of CG from Inertial frame origin   
 theta = 60;    % [deg]    
 % Using Euler's equations to compute the angular velocity derivatives
 % The initial value of the direction cosine matrix QXx
 QXx0 = [  0,           1 , 0 ;
         -cosd(theta),  0 , sind(theta);
          sind(theta),  0 , cosd(theta) ]
 % Compute the initial unit quaternion
 q4 = 0.5*sqrt(1 + QXx0(1,1) + QXx0(2,2) + QXx0(3,3))
 q1 = (QXx0(2,3) - QXx0(3,2))/(4*q4);
 q2 = (QXx0(3,1) - QXx0(1,3))/(4*q4);
 q3 = (QXx0(1,2) - QXx0(2,1))/(4*q4);
 q0  = [q1; q2; q3; q4]  
 
 % Initial value of the angular velocity in body frame components
 w0 = [0;0;1000]*2*pi/60;          % rad/s 
% w0 and q0 used as initial conditions to find the angular velocity w and 
% quaternion q as functions of time. 
MoX = [-m*g*d*QXx0(3,2),m*g*d*QXx0(3,1),0];
Mox = QXx*MoX;
for t =0:0.01:2 
dwt = [ Mox(1)/A - (C - B)*w0(2)*w0(3)/A;
            Mox(2)/B - (A - C)*w0(3)*w0(1)/B;
                Mox(3)/C - (B - A)*w0(1)*w0(2)/A];
w(t) = w0 + dwt.*t;

Om = [0 w0(3) -w0(2) w0(1);
  -w0(3) 0 w0(1) w0(2);
  w0(2) -w0(1) 0 w0(3);
  -w0(1) -w0(2) -w0(3) 0]
dqt  = 0.5*Om*q0;
q(t) = q0 + dqt*t;


q0   = q(t);
w0   = w(t);
 
end



 
 
 